@@ -143,7 +143,6 @@ typedef struct fd
 
 static const char *env_lang = NULL;
 
-static bool Found_one = false;   /* did we find a match? */
 static bool Find_files = false;  /* just find a list of proper fortune files */
 static bool Wait = false;        /* wait desired after fortune */
 static bool Short_only = false;  /* short fortune desired */
@@ -624,40 +623,6 @@ static int add_file(int percent, const char *file, const char *dir,
         debugprint("check file fd=%d path=<%s> dir=<%s> file=<%s> percent=%d\n",
             fd, path, dir, file, percent);
         bool found = false;
-        if (!dir && (!strchr(file, '/')))
-        {
-            if (((sp = strrchr(file, '-')) != NULL) && (strcmp(sp, "-o") == 0))
-            {
-#define CALL__add_file(dir) add_file(percent, file, dir, head, tail, parent)
-#define COND_CALL__add_file(loc_dir, dir)                                      \
-    ((!strcmp((loc_dir), (dir))) ? 0 : CALL__add_file(dir))
-                /* BSD-style '-o' offensive file suffix */
-                *sp = '\0';
-                found = CALL__add_file(LOCOFFDIR) ||
-                        COND_CALL__add_file(LOCOFFDIR, OFFDIR);
-                /* put the suffix back in for better identification later */
-                *sp = '-';
-            }
-            else if (All_forts)
-            {
-                found =
-                    (CALL__add_file(LOCFORTDIR) || CALL__add_file(LOCOFFDIR) ||
-                        COND_CALL__add_file(LOCFORTDIR, FORTDIR) ||
-                        COND_CALL__add_file(LOCOFFDIR, OFFDIR));
-            }
-            else if (Offend)
-            {
-                found = (CALL__add_file(LOCOFFDIR) ||
-                         COND_CALL__add_file(LOCOFFDIR, OFFDIR));
-            }
-            else
-            {
-                found = (CALL__add_file(LOCFORTDIR) ||
-                         COND_CALL__add_file(LOCFORTDIR, FORTDIR));
-            }
-#undef COND_CALL__add_file
-#undef CALL__add_file
-        }
         if (!found && !parent && !dir)
         { /* don't display an error when trying language specific files */
             if (env_lang)
@@ -1054,10 +1019,23 @@ static int form_file_list(char **files, int file_cnt)
                 sp = files[i];
             }
         }
+
+        /* BSD-style '-o' offensive file suffix */
+        bool offensive = false;
+        const size_t sp_len = strlen(sp);
+        if (sp_len >= 3 && sp[sp_len - 2] == '-' && sp[sp_len - 1] == 'o')
+        {
+            sp[sp_len - 2] = '\0';
+            offensive = true;
+        }
+
+        const char *fulldir = offensive ? OFFDIR : FORTDIR;
+        const char *locdir = offensive ? LOCOFFDIR : LOCFORTDIR;
+
         if (strcmp(sp, "all") == 0)
         {
-            snprintf(fullpathname, sizeof(fullpathname), "%s", FORTDIR);
-            snprintf(locpathname, sizeof(locpathname), "%s", LOCFORTDIR);
+            snprintf(fullpathname, sizeof(fullpathname), "%s", fulldir);
+            snprintf(locpathname, sizeof(locpathname), "%s", locdir);
         }
         /* if it isn't an absolute path or relative to . or ..
            make it an absolute path relative to FORTDIR */
@@ -1067,9 +1045,8 @@ static int form_file_list(char **files, int file_cnt)
                 strncmp(sp, "../", 3) != 0)
             {
                 snprintf(
-                    fullpathname, sizeof(fullpathname), "%s/%s", FORTDIR, sp);
-                snprintf(
-                    locpathname, sizeof(locpathname), "%s/%s", LOCFORTDIR, sp);
+                    fullpathname, sizeof(fullpathname), "%s/%s", fulldir, sp);
+                snprintf(locpathname, sizeof(locpathname), "%s/%s", locdir, sp);
             }
             else
             {
@@ -1098,7 +1075,7 @@ static int form_file_list(char **files, int file_cnt)
 
                 /* first try full locale */
                 snprintf(
-                    langdir, sizeof(langdir), "%s/%s/%s", FORTDIR, lang, sp);
+                    langdir, sizeof(langdir), "%s/%s/%s", fulldir, lang, sp);
                 ret = add_file(
                     percent, langdir, NULL, &File_list, &File_tail, NULL);
 
@@ -1110,7 +1087,7 @@ static int form_file_list(char **files, int file_cnt)
                     strncpy(ll, lang, 2);
                     ll[2] = '\0';
                     snprintf(
-                        langdir, sizeof(langdir), "%s/%s/%s", FORTDIR, ll, sp);
+                        langdir, sizeof(langdir), "%s/%s/%s", fulldir, ll, sp);
                     ret = add_file(
                         percent, langdir, NULL, &File_list, &File_tail, NULL);
                 }
@@ -1129,8 +1106,19 @@ static int form_file_list(char **files, int file_cnt)
                 ret = add_file(
                     percent, locpathname, NULL, &File_list, &File_tail, NULL);
             }
+            if (strncmp(fullpathname, locpathname, sizeof(fullpathname)) &&
+                strcmp(sp, "all") == 0)
+            {
+                add_file(
+                    percent, locpathname, NULL, &File_list, &File_tail, NULL);
+            }
             if (!ret)
             {
+                if (offensive)
+                {
+                    // restore -o suffix
+                    sp[sp_len - 2] = '-';
+                }
                 snprintf(locpathname, sizeof(locpathname), "%s/%s",
                     getenv("PWD"), sp);
 
@@ -1141,12 +1129,6 @@ static int form_file_list(char **files, int file_cnt)
             {
                 return false;
             }
-            if (strncmp(fullpathname, locpathname, sizeof(fullpathname)) &&
-                strcmp(sp, "all") == 0)
-            {
-                add_file(
-                    percent, locpathname, NULL, &File_list, &File_tail, NULL);
-            }
         }
         else if (!add_file(
                      percent, fullpathname, NULL, &File_list, &File_tail, NULL))
@@ -1162,9 +1144,8 @@ static int form_file_list(char **files, int file_cnt)
  */
 static void getargs(int argc, char **argv)
 {
-    bool ignore_case = false;
-
 #ifdef WITH_REGEX
+    bool ignore_case = false;
     char *pat = NULL;
 #endif
 
@@ -1183,7 +1164,7 @@ static void getargs(int argc, char **argv)
 #endif
 
     while ((ch = getopt(argc, argv,
-                "ac" DEBUG_GETOPT "efilm:n:" OFFENSIVE_GETOPT "suvw")) != EOF)
+                "ac" DEBUG_GETOPT "efhilm:n:" OFFENSIVE_GETOPT "suvw")) != EOF)
     {
         switch (ch)
         {
@@ -1245,6 +1226,7 @@ static void getargs(int argc, char **argv)
         case 'c':
             Show_filename = true;
             break;
+        case 'h':
         case '?':
         default:
             usage();
@@ -1714,7 +1696,7 @@ static int maxlen_in_list(FILEDESC *list)
  * matches_in_list
  *      Print out the matches from the files in the list.
  */
-static void matches_in_list(FILEDESC *list)
+static void matches_in_list(FILEDESC *list, bool *const Found_one_ptr)
 {
     unsigned char *sp;
     unsigned char *p; /* -allover */
@@ -1727,7 +1709,7 @@ static void matches_in_list(FILEDESC *list)
     {
         if (fp->child)
         {
-            matches_in_list(fp->child);
+            matches_in_list(fp->child, Found_one_ptr);
             continue;
         }
         DPRINTF(1, (stderr, "searching in %s\n", fp->path));
@@ -1778,7 +1760,7 @@ static void matches_in_list(FILEDESC *list)
                     {
                         fprintf(
                             stderr, "(%s)\n%c\n", fp->name, fp->tbl.str_delim);
-                        Found_one = true;
+                        (*Found_one_ptr) = true;
                         in_file = true;
                     }
                     fputs(output, stdout);
@@ -1802,15 +1784,15 @@ static void matches_in_list(FILEDESC *list)
  *      Find all the fortunes which match the pattern we've been
  * given.
  */
-static int find_matches(void)
+static bool find_matches(void)
 {
     Fort_len = maxlen_in_list(File_list);
     DPRINTF(2, (stderr, "Maximum length is %d\n", Fort_len));
     /* extra length, "%\n" is appended */
     Fortbuf = do_malloc((unsigned int)Fort_len + 10);
 
-    Found_one = false;
-    matches_in_list(File_list);
+    bool Found_one = false;
+    matches_in_list(File_list, &Found_one);
     return Found_one;
     /* NOTREACHED */
 }
@@ -1966,7 +1948,7 @@ int main(int argc, char *argv[])
 #ifdef WITH_REGEX
     if (Match)
     {
-        exit_code = (find_matches() != 0);
+        exit_code = find_matches();
         regfree(&Re_pat);
         goto cleanup;
     }
@@ -1999,7 +1981,10 @@ int main(int argc, char *argv[])
             sleep((unsigned int)mymax(Fort_len / CPERS, MINW));
         }
     }
+#ifdef WITH_REGEX
 cleanup:
+#endif
+
 #ifdef WITH_RECODE
     recode_delete_request(request);
     recode_delete_outer(outer);
